# Generated from pddl.g4 by ANTLR 4.7
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .pddlParser import pddlParser
else:
    from pddlParser import pddlParser

# This class defines a complete listener for a parse tree produced by pddlParser.
class pddlListener(ParseTreeListener):

    # Enter a parse tree produced by pddlParser#pddlDoc.
    def enterPddlDoc(self, ctx:pddlParser.PddlDocContext):
        pass

    # Exit a parse tree produced by pddlParser#pddlDoc.
    def exitPddlDoc(self, ctx:pddlParser.PddlDocContext):
        pass


    # Enter a parse tree produced by pddlParser#domain.
    def enterDomain(self, ctx:pddlParser.DomainContext):
        pass

    # Exit a parse tree produced by pddlParser#domain.
    def exitDomain(self, ctx:pddlParser.DomainContext):
        pass


    # Enter a parse tree produced by pddlParser#domainName.
    def enterDomainName(self, ctx:pddlParser.DomainNameContext):
        pass

    # Exit a parse tree produced by pddlParser#domainName.
    def exitDomainName(self, ctx:pddlParser.DomainNameContext):
        pass


    # Enter a parse tree produced by pddlParser#requireDef.
    def enterRequireDef(self, ctx:pddlParser.RequireDefContext):
        pass

    # Exit a parse tree produced by pddlParser#requireDef.
    def exitRequireDef(self, ctx:pddlParser.RequireDefContext):
        pass


    # Enter a parse tree produced by pddlParser#typesDef.
    def enterTypesDef(self, ctx:pddlParser.TypesDefContext):
        pass

    # Exit a parse tree produced by pddlParser#typesDef.
    def exitTypesDef(self, ctx:pddlParser.TypesDefContext):
        pass


    # Enter a parse tree produced by pddlParser#typedNameList.
    def enterTypedNameList(self, ctx:pddlParser.TypedNameListContext):
        pass

    # Exit a parse tree produced by pddlParser#typedNameList.
    def exitTypedNameList(self, ctx:pddlParser.TypedNameListContext):
        pass


    # Enter a parse tree produced by pddlParser#singleTypeNameList.
    def enterSingleTypeNameList(self, ctx:pddlParser.SingleTypeNameListContext):
        pass

    # Exit a parse tree produced by pddlParser#singleTypeNameList.
    def exitSingleTypeNameList(self, ctx:pddlParser.SingleTypeNameListContext):
        pass


    # Enter a parse tree produced by pddlParser#r_type.
    def enterR_type(self, ctx:pddlParser.R_typeContext):
        pass

    # Exit a parse tree produced by pddlParser#r_type.
    def exitR_type(self, ctx:pddlParser.R_typeContext):
        pass


    # Enter a parse tree produced by pddlParser#primType.
    def enterPrimType(self, ctx:pddlParser.PrimTypeContext):
        pass

    # Exit a parse tree produced by pddlParser#primType.
    def exitPrimType(self, ctx:pddlParser.PrimTypeContext):
        pass


    # Enter a parse tree produced by pddlParser#functionsDef.
    def enterFunctionsDef(self, ctx:pddlParser.FunctionsDefContext):
        pass

    # Exit a parse tree produced by pddlParser#functionsDef.
    def exitFunctionsDef(self, ctx:pddlParser.FunctionsDefContext):
        pass


    # Enter a parse tree produced by pddlParser#functionList.
    def enterFunctionList(self, ctx:pddlParser.FunctionListContext):
        pass

    # Exit a parse tree produced by pddlParser#functionList.
    def exitFunctionList(self, ctx:pddlParser.FunctionListContext):
        pass


    # Enter a parse tree produced by pddlParser#atomicFunctionSkeleton.
    def enterAtomicFunctionSkeleton(self, ctx:pddlParser.AtomicFunctionSkeletonContext):
        pass

    # Exit a parse tree produced by pddlParser#atomicFunctionSkeleton.
    def exitAtomicFunctionSkeleton(self, ctx:pddlParser.AtomicFunctionSkeletonContext):
        pass


    # Enter a parse tree produced by pddlParser#functionSymbol.
    def enterFunctionSymbol(self, ctx:pddlParser.FunctionSymbolContext):
        pass

    # Exit a parse tree produced by pddlParser#functionSymbol.
    def exitFunctionSymbol(self, ctx:pddlParser.FunctionSymbolContext):
        pass


    # Enter a parse tree produced by pddlParser#functionType.
    def enterFunctionType(self, ctx:pddlParser.FunctionTypeContext):
        pass

    # Exit a parse tree produced by pddlParser#functionType.
    def exitFunctionType(self, ctx:pddlParser.FunctionTypeContext):
        pass


    # Enter a parse tree produced by pddlParser#constantsDef.
    def enterConstantsDef(self, ctx:pddlParser.ConstantsDefContext):
        pass

    # Exit a parse tree produced by pddlParser#constantsDef.
    def exitConstantsDef(self, ctx:pddlParser.ConstantsDefContext):
        pass


    # Enter a parse tree produced by pddlParser#predicatesDef.
    def enterPredicatesDef(self, ctx:pddlParser.PredicatesDefContext):
        pass

    # Exit a parse tree produced by pddlParser#predicatesDef.
    def exitPredicatesDef(self, ctx:pddlParser.PredicatesDefContext):
        pass


    # Enter a parse tree produced by pddlParser#atomicFormulaSkeleton.
    def enterAtomicFormulaSkeleton(self, ctx:pddlParser.AtomicFormulaSkeletonContext):
        pass

    # Exit a parse tree produced by pddlParser#atomicFormulaSkeleton.
    def exitAtomicFormulaSkeleton(self, ctx:pddlParser.AtomicFormulaSkeletonContext):
        pass


    # Enter a parse tree produced by pddlParser#predicate.
    def enterPredicate(self, ctx:pddlParser.PredicateContext):
        pass

    # Exit a parse tree produced by pddlParser#predicate.
    def exitPredicate(self, ctx:pddlParser.PredicateContext):
        pass


    # Enter a parse tree produced by pddlParser#typedVariableList.
    def enterTypedVariableList(self, ctx:pddlParser.TypedVariableListContext):
        pass

    # Exit a parse tree produced by pddlParser#typedVariableList.
    def exitTypedVariableList(self, ctx:pddlParser.TypedVariableListContext):
        pass


    # Enter a parse tree produced by pddlParser#singleTypeVarList.
    def enterSingleTypeVarList(self, ctx:pddlParser.SingleTypeVarListContext):
        pass

    # Exit a parse tree produced by pddlParser#singleTypeVarList.
    def exitSingleTypeVarList(self, ctx:pddlParser.SingleTypeVarListContext):
        pass


    # Enter a parse tree produced by pddlParser#constraints.
    def enterConstraints(self, ctx:pddlParser.ConstraintsContext):
        pass

    # Exit a parse tree produced by pddlParser#constraints.
    def exitConstraints(self, ctx:pddlParser.ConstraintsContext):
        pass


    # Enter a parse tree produced by pddlParser#structureDef.
    def enterStructureDef(self, ctx:pddlParser.StructureDefContext):
        pass

    # Exit a parse tree produced by pddlParser#structureDef.
    def exitStructureDef(self, ctx:pddlParser.StructureDefContext):
        pass


    # Enter a parse tree produced by pddlParser#actionDef.
    def enterActionDef(self, ctx:pddlParser.ActionDefContext):
        pass

    # Exit a parse tree produced by pddlParser#actionDef.
    def exitActionDef(self, ctx:pddlParser.ActionDefContext):
        pass


    # Enter a parse tree produced by pddlParser#actionSymbol.
    def enterActionSymbol(self, ctx:pddlParser.ActionSymbolContext):
        pass

    # Exit a parse tree produced by pddlParser#actionSymbol.
    def exitActionSymbol(self, ctx:pddlParser.ActionSymbolContext):
        pass


    # Enter a parse tree produced by pddlParser#actionDefBody.
    def enterActionDefBody(self, ctx:pddlParser.ActionDefBodyContext):
        pass

    # Exit a parse tree produced by pddlParser#actionDefBody.
    def exitActionDefBody(self, ctx:pddlParser.ActionDefBodyContext):
        pass


    # Enter a parse tree produced by pddlParser#precondition.
    def enterPrecondition(self, ctx:pddlParser.PreconditionContext):
        pass

    # Exit a parse tree produced by pddlParser#precondition.
    def exitPrecondition(self, ctx:pddlParser.PreconditionContext):
        pass


    # Enter a parse tree produced by pddlParser#goalDesc.
    def enterGoalDesc(self, ctx:pddlParser.GoalDescContext):
        pass

    # Exit a parse tree produced by pddlParser#goalDesc.
    def exitGoalDesc(self, ctx:pddlParser.GoalDescContext):
        pass


    # Enter a parse tree produced by pddlParser#fComp.
    def enterFComp(self, ctx:pddlParser.FCompContext):
        pass

    # Exit a parse tree produced by pddlParser#fComp.
    def exitFComp(self, ctx:pddlParser.FCompContext):
        pass


    # Enter a parse tree produced by pddlParser#atomicTermFormula.
    def enterAtomicTermFormula(self, ctx:pddlParser.AtomicTermFormulaContext):
        pass

    # Exit a parse tree produced by pddlParser#atomicTermFormula.
    def exitAtomicTermFormula(self, ctx:pddlParser.AtomicTermFormulaContext):
        pass


    # Enter a parse tree produced by pddlParser#term.
    def enterTerm(self, ctx:pddlParser.TermContext):
        pass

    # Exit a parse tree produced by pddlParser#term.
    def exitTerm(self, ctx:pddlParser.TermContext):
        pass


    # Enter a parse tree produced by pddlParser#durativeActionDef.
    def enterDurativeActionDef(self, ctx:pddlParser.DurativeActionDefContext):
        pass

    # Exit a parse tree produced by pddlParser#durativeActionDef.
    def exitDurativeActionDef(self, ctx:pddlParser.DurativeActionDefContext):
        pass


    # Enter a parse tree produced by pddlParser#daDefBody.
    def enterDaDefBody(self, ctx:pddlParser.DaDefBodyContext):
        pass

    # Exit a parse tree produced by pddlParser#daDefBody.
    def exitDaDefBody(self, ctx:pddlParser.DaDefBodyContext):
        pass


    # Enter a parse tree produced by pddlParser#daGD.
    def enterDaGD(self, ctx:pddlParser.DaGDContext):
        pass

    # Exit a parse tree produced by pddlParser#daGD.
    def exitDaGD(self, ctx:pddlParser.DaGDContext):
        pass


    # Enter a parse tree produced by pddlParser#prefTimedGD.
    def enterPrefTimedGD(self, ctx:pddlParser.PrefTimedGDContext):
        pass

    # Exit a parse tree produced by pddlParser#prefTimedGD.
    def exitPrefTimedGD(self, ctx:pddlParser.PrefTimedGDContext):
        pass


    # Enter a parse tree produced by pddlParser#timedGD.
    def enterTimedGD(self, ctx:pddlParser.TimedGDContext):
        pass

    # Exit a parse tree produced by pddlParser#timedGD.
    def exitTimedGD(self, ctx:pddlParser.TimedGDContext):
        pass


    # Enter a parse tree produced by pddlParser#timeSpecifier.
    def enterTimeSpecifier(self, ctx:pddlParser.TimeSpecifierContext):
        pass

    # Exit a parse tree produced by pddlParser#timeSpecifier.
    def exitTimeSpecifier(self, ctx:pddlParser.TimeSpecifierContext):
        pass


    # Enter a parse tree produced by pddlParser#interval.
    def enterInterval(self, ctx:pddlParser.IntervalContext):
        pass

    # Exit a parse tree produced by pddlParser#interval.
    def exitInterval(self, ctx:pddlParser.IntervalContext):
        pass


    # Enter a parse tree produced by pddlParser#derivedDef.
    def enterDerivedDef(self, ctx:pddlParser.DerivedDefContext):
        pass

    # Exit a parse tree produced by pddlParser#derivedDef.
    def exitDerivedDef(self, ctx:pddlParser.DerivedDefContext):
        pass


    # Enter a parse tree produced by pddlParser#fExp.
    def enterFExp(self, ctx:pddlParser.FExpContext):
        pass

    # Exit a parse tree produced by pddlParser#fExp.
    def exitFExp(self, ctx:pddlParser.FExpContext):
        pass


    # Enter a parse tree produced by pddlParser#fExp2.
    def enterFExp2(self, ctx:pddlParser.FExp2Context):
        pass

    # Exit a parse tree produced by pddlParser#fExp2.
    def exitFExp2(self, ctx:pddlParser.FExp2Context):
        pass


    # Enter a parse tree produced by pddlParser#fHead.
    def enterFHead(self, ctx:pddlParser.FHeadContext):
        pass

    # Exit a parse tree produced by pddlParser#fHead.
    def exitFHead(self, ctx:pddlParser.FHeadContext):
        pass


    # Enter a parse tree produced by pddlParser#effect.
    def enterEffect(self, ctx:pddlParser.EffectContext):
        pass

    # Exit a parse tree produced by pddlParser#effect.
    def exitEffect(self, ctx:pddlParser.EffectContext):
        pass


    # Enter a parse tree produced by pddlParser#cEffect.
    def enterCEffect(self, ctx:pddlParser.CEffectContext):
        pass

    # Exit a parse tree produced by pddlParser#cEffect.
    def exitCEffect(self, ctx:pddlParser.CEffectContext):
        pass


    # Enter a parse tree produced by pddlParser#pEffect.
    def enterPEffect(self, ctx:pddlParser.PEffectContext):
        pass

    # Exit a parse tree produced by pddlParser#pEffect.
    def exitPEffect(self, ctx:pddlParser.PEffectContext):
        pass


    # Enter a parse tree produced by pddlParser#condEffect.
    def enterCondEffect(self, ctx:pddlParser.CondEffectContext):
        pass

    # Exit a parse tree produced by pddlParser#condEffect.
    def exitCondEffect(self, ctx:pddlParser.CondEffectContext):
        pass


    # Enter a parse tree produced by pddlParser#binaryOp.
    def enterBinaryOp(self, ctx:pddlParser.BinaryOpContext):
        pass

    # Exit a parse tree produced by pddlParser#binaryOp.
    def exitBinaryOp(self, ctx:pddlParser.BinaryOpContext):
        pass


    # Enter a parse tree produced by pddlParser#binaryComp.
    def enterBinaryComp(self, ctx:pddlParser.BinaryCompContext):
        pass

    # Exit a parse tree produced by pddlParser#binaryComp.
    def exitBinaryComp(self, ctx:pddlParser.BinaryCompContext):
        pass


    # Enter a parse tree produced by pddlParser#assignOp.
    def enterAssignOp(self, ctx:pddlParser.AssignOpContext):
        pass

    # Exit a parse tree produced by pddlParser#assignOp.
    def exitAssignOp(self, ctx:pddlParser.AssignOpContext):
        pass


    # Enter a parse tree produced by pddlParser#durationConstraint.
    def enterDurationConstraint(self, ctx:pddlParser.DurationConstraintContext):
        pass

    # Exit a parse tree produced by pddlParser#durationConstraint.
    def exitDurationConstraint(self, ctx:pddlParser.DurationConstraintContext):
        pass


    # Enter a parse tree produced by pddlParser#simpleDurationConstraint.
    def enterSimpleDurationConstraint(self, ctx:pddlParser.SimpleDurationConstraintContext):
        pass

    # Exit a parse tree produced by pddlParser#simpleDurationConstraint.
    def exitSimpleDurationConstraint(self, ctx:pddlParser.SimpleDurationConstraintContext):
        pass


    # Enter a parse tree produced by pddlParser#durOp.
    def enterDurOp(self, ctx:pddlParser.DurOpContext):
        pass

    # Exit a parse tree produced by pddlParser#durOp.
    def exitDurOp(self, ctx:pddlParser.DurOpContext):
        pass


    # Enter a parse tree produced by pddlParser#durValue.
    def enterDurValue(self, ctx:pddlParser.DurValueContext):
        pass

    # Exit a parse tree produced by pddlParser#durValue.
    def exitDurValue(self, ctx:pddlParser.DurValueContext):
        pass


    # Enter a parse tree produced by pddlParser#daEffect.
    def enterDaEffect(self, ctx:pddlParser.DaEffectContext):
        pass

    # Exit a parse tree produced by pddlParser#daEffect.
    def exitDaEffect(self, ctx:pddlParser.DaEffectContext):
        pass


    # Enter a parse tree produced by pddlParser#timedEffect.
    def enterTimedEffect(self, ctx:pddlParser.TimedEffectContext):
        pass

    # Exit a parse tree produced by pddlParser#timedEffect.
    def exitTimedEffect(self, ctx:pddlParser.TimedEffectContext):
        pass


    # Enter a parse tree produced by pddlParser#fAssignDA.
    def enterFAssignDA(self, ctx:pddlParser.FAssignDAContext):
        pass

    # Exit a parse tree produced by pddlParser#fAssignDA.
    def exitFAssignDA(self, ctx:pddlParser.FAssignDAContext):
        pass


    # Enter a parse tree produced by pddlParser#fExpDA.
    def enterFExpDA(self, ctx:pddlParser.FExpDAContext):
        pass

    # Exit a parse tree produced by pddlParser#fExpDA.
    def exitFExpDA(self, ctx:pddlParser.FExpDAContext):
        pass


    # Enter a parse tree produced by pddlParser#assignOpT.
    def enterAssignOpT(self, ctx:pddlParser.AssignOpTContext):
        pass

    # Exit a parse tree produced by pddlParser#assignOpT.
    def exitAssignOpT(self, ctx:pddlParser.AssignOpTContext):
        pass


    # Enter a parse tree produced by pddlParser#problem.
    def enterProblem(self, ctx:pddlParser.ProblemContext):
        pass

    # Exit a parse tree produced by pddlParser#problem.
    def exitProblem(self, ctx:pddlParser.ProblemContext):
        pass


    # Enter a parse tree produced by pddlParser#problemDecl.
    def enterProblemDecl(self, ctx:pddlParser.ProblemDeclContext):
        pass

    # Exit a parse tree produced by pddlParser#problemDecl.
    def exitProblemDecl(self, ctx:pddlParser.ProblemDeclContext):
        pass


    # Enter a parse tree produced by pddlParser#problemDomain.
    def enterProblemDomain(self, ctx:pddlParser.ProblemDomainContext):
        pass

    # Exit a parse tree produced by pddlParser#problemDomain.
    def exitProblemDomain(self, ctx:pddlParser.ProblemDomainContext):
        pass


    # Enter a parse tree produced by pddlParser#objectDecl.
    def enterObjectDecl(self, ctx:pddlParser.ObjectDeclContext):
        pass

    # Exit a parse tree produced by pddlParser#objectDecl.
    def exitObjectDecl(self, ctx:pddlParser.ObjectDeclContext):
        pass


    # Enter a parse tree produced by pddlParser#init.
    def enterInit(self, ctx:pddlParser.InitContext):
        pass

    # Exit a parse tree produced by pddlParser#init.
    def exitInit(self, ctx:pddlParser.InitContext):
        pass


    # Enter a parse tree produced by pddlParser#initEl.
    def enterInitEl(self, ctx:pddlParser.InitElContext):
        pass

    # Exit a parse tree produced by pddlParser#initEl.
    def exitInitEl(self, ctx:pddlParser.InitElContext):
        pass


    # Enter a parse tree produced by pddlParser#nameLiteral.
    def enterNameLiteral(self, ctx:pddlParser.NameLiteralContext):
        pass

    # Exit a parse tree produced by pddlParser#nameLiteral.
    def exitNameLiteral(self, ctx:pddlParser.NameLiteralContext):
        pass


    # Enter a parse tree produced by pddlParser#atomicNameFormula.
    def enterAtomicNameFormula(self, ctx:pddlParser.AtomicNameFormulaContext):
        pass

    # Exit a parse tree produced by pddlParser#atomicNameFormula.
    def exitAtomicNameFormula(self, ctx:pddlParser.AtomicNameFormulaContext):
        pass


    # Enter a parse tree produced by pddlParser#goal.
    def enterGoal(self, ctx:pddlParser.GoalContext):
        pass

    # Exit a parse tree produced by pddlParser#goal.
    def exitGoal(self, ctx:pddlParser.GoalContext):
        pass


    # Enter a parse tree produced by pddlParser#probConstraints.
    def enterProbConstraints(self, ctx:pddlParser.ProbConstraintsContext):
        pass

    # Exit a parse tree produced by pddlParser#probConstraints.
    def exitProbConstraints(self, ctx:pddlParser.ProbConstraintsContext):
        pass


    # Enter a parse tree produced by pddlParser#prefConGD.
    def enterPrefConGD(self, ctx:pddlParser.PrefConGDContext):
        pass

    # Exit a parse tree produced by pddlParser#prefConGD.
    def exitPrefConGD(self, ctx:pddlParser.PrefConGDContext):
        pass


    # Enter a parse tree produced by pddlParser#metricSpec.
    def enterMetricSpec(self, ctx:pddlParser.MetricSpecContext):
        pass

    # Exit a parse tree produced by pddlParser#metricSpec.
    def exitMetricSpec(self, ctx:pddlParser.MetricSpecContext):
        pass


    # Enter a parse tree produced by pddlParser#optimization.
    def enterOptimization(self, ctx:pddlParser.OptimizationContext):
        pass

    # Exit a parse tree produced by pddlParser#optimization.
    def exitOptimization(self, ctx:pddlParser.OptimizationContext):
        pass


    # Enter a parse tree produced by pddlParser#metricFExp.
    def enterMetricFExp(self, ctx:pddlParser.MetricFExpContext):
        pass

    # Exit a parse tree produced by pddlParser#metricFExp.
    def exitMetricFExp(self, ctx:pddlParser.MetricFExpContext):
        pass


    # Enter a parse tree produced by pddlParser#conGD.
    def enterConGD(self, ctx:pddlParser.ConGDContext):
        pass

    # Exit a parse tree produced by pddlParser#conGD.
    def exitConGD(self, ctx:pddlParser.ConGDContext):
        pass


    # Enter a parse tree produced by pddlParser#name.
    def enterName(self, ctx:pddlParser.NameContext):
        pass

    # Exit a parse tree produced by pddlParser#name.
    def exitName(self, ctx:pddlParser.NameContext):
        pass





from planning_sat.encoder import PlanningProblemEncoder, Clause, Operator
from pddlpy.pddl import Operator as Op


formulas = [Clause(('atl', 'rob', 'loc1', '0')),
            Clause(('not', 'atl', 'rob', 'loc2', '0')),
            Clause(('atl', 'rob', 'loc2', '1')),
            Clause((
                'not',
                Op('move', {'?r': 'rob', '?from': 'loc1', '?to': 'loc2'}),
                '0')).add(('atl', 'rob', 'loc1', '0'), Operator.OR),
            Clause((
                'not',
                Op('move', {'?r': 'rob', '?from': 'loc1', '?to': 'loc2'}),
                '0')).add(('atl', 'rob', 'loc2', '1'), Operator.OR),
            Clause((
                'not',
                Op('move', {'?r': 'rob', '?from': 'loc1', '?to': 'loc2'}),
                '0')).add(('not', 'atl', 'rob', 'loc1', '1'), Operator.OR),
            Clause((
                'not',
                Op('move', {'?r': 'rob', '?from': 'loc2', '?to': 'loc1'}),
                '0')).add(('atl', 'rob', 'loc2', '0'), Operator.OR),
            Clause((
                'not',
                Op('move', {'?r': 'rob', '?from': 'loc2', '?to': 'loc1'}),
                '0')).add(('atl', 'rob', 'loc1', '1'), Operator.OR),
            Clause((
                'not',
                Op('move', {'?r': 'rob', '?from': 'loc2', '?to': 'loc1'}),
                '0')).add(('not', 'atl', 'rob', 'loc2', '1'), Operator.OR),
            Clause(('atl', 'rob', 'loc1', '0')).add(
                ('not', 'atl', 'rob', 'loc1', '1'), Operator.OR).add(
                (Op('move', {'?r': 'rob', '?from': 'loc2', '?to': 'loc1'}), '0'
                 ), Operator.OR),
            Clause(('not', 'atl', 'rob', 'loc1', '0')).add(
                ('atl', 'rob', 'loc1', '1'), Operator.OR).add(
                (Op('move', {'?r': 'rob', '?from': 'loc1', '?to': 'loc2'}), '0'
                 ), Operator.OR),
            Clause(('atl', 'rob', 'loc2', '0')).add(
                ('not', 'atl', 'rob', 'loc2', '1'), Operator.OR).add(
                (Op('move', {'?r': 'rob', '?from': 'loc1', '?to': 'loc2'}), '0'
                 ), Operator.OR),
            Clause(('not', 'atl', 'rob', 'loc2', '0')).add(
                ('atl', 'rob', 'loc2', '1'), Operator.OR).add(
                (Op('move', {'?r': 'rob', '?from': 'loc2', '?to': 'loc1'}), '0'
                 ), Operator.OR),
            Clause(('not',
                    Op('move', {'?r': 'rob', '?from': 'loc1', '?to': 'loc2'}),
                    '0')).add(('not', Op('move', {'?r': 'rob', '?from': 'loc2',
                                                  '?to': 'loc1'}), '0'),
                              Operator.OR)
            ]


class TestEncoder:

    def test_encoder(self):
        encoded_pp = PlanningProblemEncoder("domain/simple-domain.pddl",
                                            "domain/simple-problem.pddl")
        pp = encoded_pp.propositional_formulas
        assert len(pp) == len(formulas)



"""PDDL Adapter

Description:
    The adaptation layer (wrapper) of pddlpy module.

License:
    Copyright 2021 Debby Nirwan

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""
from pddlpy import DomainProblem, Operator, Atom
from typing import Set, Tuple, List
import itertools


class PlanningProblem(object):

    def __init__(self, dom_file: str, problem_file: str):
        self._domain_file = dom_file
        self._problem_file = problem_file
        self._domain_problem = DomainProblem(self._domain_file,
                                             self._problem_file)
        self._initial_state = self._to_set_of_tuples(self._domain_problem.
                                                     initialstate())
        self._goal_state = self._to_set_of_tuples(self._domain_problem.goals())
        self._actions = self._get_ground_operators()
        self._formulas = self._get_ground_formulas()

    @staticmethod
    def _type_symbols(variable_type, world_objects: dict):
        # if variable type is found in the world objects,
        # return list of object names, such as robr, robq
        return (k for k, v in world_objects.items() if v == variable_type)

    def _instantiate(self, variables, world_objects: dict):
        variable_ground_space = []
        for variable_name, variable_type in variables:
            c = []
            for symbol in self._type_symbols(variable_type, world_objects):
                c.append((variable_name, symbol))
            variable_ground_space.append(c)
        return itertools.product(*variable_ground_space)

    def _get_ground_operators(self) -> List[Operator]:
        ground_operators = []
        for operator in self._domain_problem.operators():
            op = self._domain_problem.domain.operators[operator]
            for ground in self._instantiate(op.variable_list.items(),
                                            self._domain_problem.
                                            worldobjects()):
                st = dict(ground)
                gop = Operator(operator)
                gop.variable_list = st
                gop.precondition_pos = set(
                    [a.ground(st) for a in op.precondition_pos])
                gop.precondition_neg = set(
                    [a.ground(st) for a in op.precondition_neg])
                gop.effect_pos = set([a.ground(st) for a in op.effect_pos])
                gop.effect_neg = set([a.ground(st) for a in op.effect_neg])
                ground_operators.append(gop)
        return ground_operators

    def _get_ground_formulas(self) -> List[tuple]:
        ground_formulas = []
        for predicate in self._domain_problem.domain.predicates:
            if predicate.name == 'adjacent':
                continue
            for ground in self._instantiate(predicate.variables.items(),
                                            self._domain_problem.
                                            worldobjects()):
                st = dict(ground)
                pred = [predicate.name]
                for k, v in st.items():
                    pred.append(v)
                ground_formulas.append(tuple(pred))
        return ground_formulas

    @staticmethod
    def _to_set_of_tuples(state: Set[Atom]) -> Set[Tuple]:
        set_of_tuples = set()
        for atom in state:
            tup = tuple(atom.predicate)
            set_of_tuples.add(tup)
        return set_of_tuples

    @property
    def initial_state(self):
        return self._initial_state

    @property
    def goal_state(self):
        return self._goal_state

    @property
    def actions(self):
        return self._actions

    @property
    def fluents(self):
        return self._formulas



#!/usr/bin/python
# -*- coding: utf-8 -*-

#
# Copyright 2015 Hernán M. Foffani
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#

from antlr4 import *
from .pddlLexer import pddlLexer
from .pddlParser import pddlParser
from .pddlListener import pddlListener

import itertools


class Atom():
    def __init__(self, predicate):
        self.predicate = predicate

    def __repr__(self):
        return str(tuple(self.predicate))

    def ground(self, varvals):
        g = [varvals[v] if v in varvals else v for v in self.predicate]
        return tuple(g)


class Predicate(object):

    def __init__(self, name):
        self.name = name
        self.variables: dict = {}

    def add_variable(self, name, var_type):
        self.variables[name] = var_type


class Scope():
    def __init__(self):
        self.atoms = []
        self.negatoms = []

    def addatom(self, atom):
        self.atoms.append(atom)

    def addnegatom(self, atom):
        self.negatoms.append(atom)


class Obj():
    def __init__(self):
        self.variable_list = {}


class Operator():
    """Represents and action. Can be grounded or ungrounded.
    Ungrounded operators have a '?' in names (unbound variables).
    Attributes:

        operator_name -- the name of operator (action in the domain.)
        variable_list -- a dictionary of key-value pairs where the key
                         is the variable name (with the '?') and the
                         value is the value of it when the operator is
                         grounded.
        precondition_pos -- a set of atoms corresponding to the
                            positive preconditions.
        precondition_neg -- a set of atoms corresponding to the
                            negative preconditions.
        effect_pos -- a set of atoms to add.
        effect_neg -- a set of atoms to delete.
    """

    def __init__(self, name, var_list=None):
        if var_list is None:
            var_list = {}
        self.operator_name = name
        self.variable_list = var_list
        self.precondition_pos = set()
        self.precondition_neg = set()
        self.effect_pos = set()
        self.effect_neg = set()

    def __eq__(self, other):
        if not isinstance(other, Operator):
            return False
        if self.operator_name == other.operator_name and \
                self.variable_list == other.variable_list:
            return True
        else:
            return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def __hash__(self):
        key = (self.operator_name,) + tuple(self.variable_list)
        return hash(key)

    def __repr__(self):
        return f"{self.operator_name}({list(self.variable_list.values())})"


class DomainListener(pddlListener):
    def __init__(self):
        self.typesdef = False
        self.objects = {}
        self.operators = {}
        self.scopes = []
        self.negativescopes = []
        self.predicates = []

    def enterActionDef(self, ctx):
        opname = ctx.actionSymbol().getText()
        opvars = {}
        self.scopes.append(Operator(opname))

    def exitActionDef(self, ctx):
        action = self.scopes.pop()
        self.operators[action.operator_name] = action

    def enterPredicatesDef(self, ctx):
        self.scopes.append(Operator(None))
        list_of_predicates = ctx.atomicFormulaSkeleton()
        for predicate_ctx in list_of_predicates:
            name = predicate_ctx.predicate().getText()
            pred = Predicate(name)
            var_list_ctx = predicate_ctx.typedVariableList()
            for v in var_list_ctx.VARIABLE():
                vname = v.getText()
            for vs in var_list_ctx.singleTypeVarList():
                t = vs.r_type().getText()
                for v in vs.VARIABLE():
                    vname = v.getText()
                    pred.add_variable(vname, t)
            self.predicates.append(pred)

    def exitPredicatesDef(self, ctx):
        dummyop = self.scopes.pop()

    def enterTypesDef(self, ctx):
        self.scopes.append(Obj())

    def exitTypesDef(self, ctx):
        self.typesdef = True
        self.scopes.pop()

    def enterTypedVariableList(self, ctx):
        # print("-> tvar")
        for v in ctx.VARIABLE():
            vname = v.getText()
            self.scopes[-1].variable_list[v.getText()] = None
        for vs in ctx.singleTypeVarList():
            t = vs.r_type().getText()
            for v in vs.VARIABLE():
                vname = v.getText()
                self.scopes[-1].variable_list[vname] = t

    def enterAtomicTermFormula(self, ctx):
        # print("-> terf")
        neg = self.negativescopes[-1]
        pred = []
        for c in ctx.getChildren():
            n = c.getText()
            if n == '(' or n == ')':
                continue
            pred.append(n)
        scope = self.scopes[-1]
        if not neg:
            scope.addatom(Atom(pred))
        else:
            scope.addnegatom(Atom(pred))

    def enterPrecondition(self, ctx):
        self.scopes.append(Scope())

    def exitPrecondition(self, ctx):
        scope = self.scopes.pop()
        self.scopes[-1].precondition_pos = set(scope.atoms)
        self.scopes[-1].precondition_neg = set(scope.negatoms)

    def enterEffect(self, ctx):
        self.scopes.append(Scope())

    def exitEffect(self, ctx):
        scope = self.scopes.pop()
        self.scopes[-1].effect_pos = set(scope.atoms)
        self.scopes[-1].effect_neg = set(scope.negatoms)

    def enterGoalDesc(self, ctx):
        negscope = bool(self.negativescopes and self.negativescopes[-1])
        for c in ctx.getChildren():
            if c.getText() == 'not':
                negscope = True
                break
        self.negativescopes.append(negscope)

    def exitGoalDesc(self, ctx):
        self.negativescopes.pop()

    def enterPEffect(self, ctx):
        negscope = False
        for c in ctx.getChildren():
            if c.getText() == 'not':
                negscope = True
                break
        self.negativescopes.append(negscope)

    def exitPEffect(self, ctx):
        self.negativescopes.pop()

    def enterTypedNameList(self, ctx):
        # print("-> tnam")
        for v in ctx.name():
            vname = v.getText()
            self.scopes[-1].variable_list[v.getText()] = None
        for vs in ctx.singleTypeNameList():
            t = vs.r_type().getText()
            for v in vs.name():
                vname = v.getText()
                self.scopes[-1].variable_list[vname] = t

    def enterConstantsDef(self, ctx):
        self.scopes.append(Obj())

    def exitConstantsDef(self, ctx):
        scope = self.scopes.pop()
        self.objects = scope.variable_list

    def exitDomain(self, ctx):
        if not self.objects and not self.typesdef:
            vs = set()
            for opn, oper in self.operators.items():
                alls = oper.precondition_pos | oper.precondition_neg | oper.effect_pos | oper.effect_neg
                for a in alls:
                    for s in a.predicate:
                        if s[0] != '?':
                            vs.add((s, None))
            self.objects = dict(vs)


class ProblemListener(pddlListener):

    def __init__(self):
        self.objects = {}
        self.initialstate = []
        self.goals = []
        self.scopes = []

    def enterInit(self, ctx):
        self.scopes.append(Scope())

    def exitInit(self, ctx):
        self.initialstate = set(self.scopes.pop().atoms)

    def enterGoal(self, ctx):
        self.scopes.append(Scope())

    def exitGoal(self, ctx):
        self.goals = set(self.scopes.pop().atoms)

    def enterAtomicNameFormula(self, ctx):
        pred = []
        for c in ctx.getChildren():
            n = c.getText()
            if n == '(' or n == ')':
                continue
            pred.append(n)
        scope = self.scopes[-1]
        scope.addatom(Atom(pred))

    def enterAtomicTermFormula(self, ctx):
        # with a NOT!
        pred = []
        for c in ctx.getChildren():
            n = c.getText()
            if n == '(' or n == ')':
                continue
            pred.append(n)
        scope = self.scopes[-1]
        scope.addatom(Atom(pred))

    def enterTypedNameList(self, ctx):
        for v in ctx.name():
            vname = v.getText()
            self.scopes[-1].variable_list[v.getText()] = None
        for vs in ctx.singleTypeNameList():
            t = vs.r_type().getText()
            for v in vs.name():
                vname = v.getText()
                self.scopes[-1].variable_list[vname] = t

    def enterObjectDecl(self, ctx):
        self.scopes.append(Obj())

    def exitObjectDecl(self, ctx):
        scope = self.scopes.pop()
        self.objects = scope.variable_list

    def exitProblem(self, ctx):
        if not self.objects:
            vs = set()
            for a in self.initialstate:
                for s in a.predicate:
                    vs.add((s, None))
            for a in self.goals:
                for s in a.predicate:
                    vs.add((s, None))
            self.objects = dict(vs)


class DomainProblem():

    def __init__(self, domainfile, problemfile):
        """Parses a PDDL domain and problem files and
        returns an object representing them.

        domainfile -- path for the PDDL domain file
        problemfile -- path for the PDDL problem file
        """
        # domain
        inp = FileStream(domainfile)
        lexer = pddlLexer(inp)
        stream = CommonTokenStream(lexer)
        parser = pddlParser(stream)
        tree = parser.domain()
        self.domain = DomainListener()
        walker = ParseTreeWalker()
        walker.walk(self.domain, tree)
        # problem
        inp = FileStream(problemfile)
        lexer = pddlLexer(inp)
        stream = CommonTokenStream(lexer)
        parser = pddlParser(stream)
        tree = parser.problem()
        self.problem = ProblemListener()
        walker = ParseTreeWalker()
        walker.walk(self.problem, tree)
        # variable ground space
        self.vargroundspace = []

    def operators(self):
        """Returns an iterator of the names of the actions defined in
        the domain file.
        """
        return self.domain.operators.keys()

    def ground_operator(self, op_name):
        """Returns an interator of Operator instances. Each item of the iterator
        is a grounded instance.

        returns -- An iterator of Operator instances.
        """
        op = self.domain.operators[op_name]
        for ground in self._instantiate(op.variable_list.items()):
            st = dict(ground)
            gop = Operator(op_name)
            gop.variable_list = st
            gop.precondition_pos = set(
                [a.ground(st) for a in op.precondition_pos])
            gop.precondition_neg = set(
                [a.ground(st) for a in op.precondition_neg])
            gop.effect_pos = set([a.ground(st) for a in op.effect_pos])
            gop.effect_neg = set([a.ground(st) for a in op.effect_neg])
            yield gop

    def _typesymbols(self, t):
        return (k for k, v in self.worldobjects().items() if v == t)

    def _instantiate(self, variables):
        if not self.vargroundspace:
            for vname, t in variables:
                c = []
                for symb in self._typesymbols(t):
                    c.append((vname, symb))
                self.vargroundspace.append(c)
        return itertools.product(*self.vargroundspace)

    def initialstate(self):
        """Returns a set of atoms (tuples of strings) corresponding to the intial
        state defined in the problem file.
        """
        return self.problem.initialstate

    def goals(self):
        """Returns a set of atoms (tuples of strings) corresponding to the goals
        defined in the problem file.
        """
        return self.problem.goals

    def worldobjects(self):
        """Returns a dictionary of key value pairs where the key is the name of
        an object and the value is it's type (None in case is untyped.)
        """
        return dict(self.domain.objects.items() | self.problem.objects.items())


if __name__ == '__main__':
    pass
